@@ -3,14 +3,15 @@ package Solaris::Disk::Mnttab;
 use strict;
 use warnings;
 use Carp;
+
 #use vars qw( %Device2MountPoint %MountPoint2Device );
 
 #require Exporter;
 #our %EXPORT_TAGS = ( 'all' => [ qw( PartType PartFlag ) ] );
 #our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
-our $VERSION = "0.02";
-my $mnttabSource = '/etc/mnttab';
+our $VERSION = "0.03";
+my $mnttabSource  = '/etc/mnttab';
 my $swaptabSource = '/sbin/swap -l |';
 
 my @validfs = qw( ufs vxfs proc tmpfs );
@@ -50,6 +51,7 @@ Initialise and read tables, from optional sources
                                       );
 
 =cut
+
 sub new {
     my ( $class, @args ) = @_;
     $class = ref($class) || $class;
@@ -69,19 +71,20 @@ sub new {
             $i++;
         }
     }
+
     # shouldn't be anything left in @args
-    croak "Unknown parameter(s): @args" if @args;
+    warn "Unknown parameter(s): @args" if @args;
 
     bless $self, $class;
 
     if ( defined $parms{init} && $parms{init} ) {
-        if ( defined($parms{mnttab})) {
-            $self->readmtab(mnttab => $parms{mnttab});
+        if ( defined( $parms{mnttab} ) ) {
+            $self->readmtab( mnttab => $parms{mnttab} );
         }
         else {
             $self->readmtab();
         }
-        if ( defined($parms{swaptab})) {
+        if ( defined( $parms{swaptab} ) ) {
             $self->readstab( swaptab => $parms{swaptab} );
         }
         else {
@@ -101,13 +104,13 @@ specified, the source given by the C<mnttab> argument.
   $mnttab->readmtab( mnttab => 'mymnttabdump.txt');
 
 =cut
-sub readmtab
-{
-    my ($self, @args) = @_;
+
+sub readmtab {
+    my ( $self, @args ) = @_;
 
     my %parms;
     my $i = 0;
-    my $re = join "|", qw( mnttab mnttabdir );
+    my $re = join "|", qw( mnttab );	# mnttabdir 
     $re = qr/^(?:$re)$/;
     while ( $i < @args ) {
         if ( $args[$i] =~ $re ) {
@@ -119,41 +122,51 @@ sub readmtab
         }
     }
 
-    croak "You cannot specify both mnttab and mnttabdir parameters"
-      if (defined($parms{mnttab}) && defined($parms{mnttabdir} ) );
+    # shouldn't be anything left in @args
+    warn "Unknown parameter(s): @args" if @args;
 
-    my $source = defined($parms{mnttab})
-                       ? $parms{mnttab}
-                       : defined( $parms{mnttabdir} )
-                           ? $parms{mnttabdir}.'/mnttab.txt'
-                           : $mnttabSource;
-    open MTAB, $source
-      or croak "Can't open mptab source $source";
+    # croak "You cannot specify both mnttab and mnttabdir parameters"
+    #   if ( defined( $parms{mnttab} ) && defined( $parms{mnttabdir} ) );
 
-    my $validfs = join '|', @validfs;
-    $validfs = qr/^(?:$validfs)$/;
-    while (<MTAB>) {
-        chomp;
-        my ( $dev, $mp, $fstype, $opts, $inode );
-        ( $dev, $mp, $fstype, $opts, $inode ) = split /\s+/;
+    my $source =
+        defined( $parms{mnttab} )    ? $parms{mnttab}
+#      : defined( $parms{mnttabdir} ) ? $parms{mnttabdir} . '/mnttab.txt'
+      : $mnttabSource;
 
-        if ( $fstype =~ $validfs ) {
-            $dev =~ s!.*\/!!;
-            $self->{dev2mp}{$dev} = $mp;
-            $self->{mp2dev}{$mp}{device}  = $dev;
-            my %options = map { $_ => 1 }  split /,/, $opts;
-            foreach (keys %options) {
-                delete($options{$_}) if $_ eq '';
+    if ( open MTAB, $source ) {
+        my $validfs = join '|', @validfs;
+        $validfs = qr/^(?:$validfs)$/;
+        while (<MTAB>) {
+            chomp;
+            my ( $dev, $mp, $fstype, $opts, $inode );
+            ( $dev, $mp, $fstype, $opts, $inode ) = split /\s+/;
+            next if !defined($mp);
+            next if !defined($fstype);
+            next if !defined($opts);
+            next if !defined($inode);
+
+
+            if ( $fstype =~ $validfs ) {
+                $dev =~ s!.*\/!!;
+                $self->{dev2mp}{$dev} = $mp;
+                $self->{mp2dev}{$mp}{device} = $dev;
+                my %options = map { $_ => 1 } split /,/, $opts;
+                foreach ( keys %options ) {
+                    delete( $options{$_} ) if $_ eq '';
+                }
+                @{ $self->{mp2dev}{$mp}{options} } = sort keys %options;
+                $self->{mp2dev}{$mp}{inode}  = $inode;
+                $self->{mp2dev}{$mp}{fstype} = $fstype;
             }
-            @{$self->{mp2dev}{$mp}{options}} = sort keys %options;
-            $self->{mp2dev}{$mp}{inode}   = $inode;
-            $self->{mp2dev}{$mp}{fstype}  = $fstype;
         }
+        close MTAB
+          or warn "Can't close mnttab source $source";
+    }
+    else {
+        warn "Can't open mnttab source $source";
     }
-    close MTAB;
 }
 
-
 =head2 C<readstab>
 
 The C<readstab> method allows one to (re-)read the swap table, as given by
@@ -165,13 +178,13 @@ If specified, the source given by the C<swaptab> argument is used instead.
   $mnttab->readstab( swaptab => 'myswap-l.txt');
 
 =cut
-sub readstab
-{
-    my ($self, @args) = @_;
+
+sub readstab {
+    my ( $self, @args ) = @_;
 
     my %parms;
     my $i = 0;
-    my $re = join "|", qw( swaptab swaptabdir );
+    my $re = join "|", qw( swaptab );	# swaptabdir 
     $re = qr/^(?:$re)$/;
     while ( $i < @args ) {
         if ( $args[$i] =~ $re ) {
@@ -183,39 +196,48 @@ sub readstab
         }
     }
 
-    croak "You cannot specify both swaptab and swaptabdir parameters"
-      if (defined($parms{swaptab}) && defined($parms{swaptabdir}) );
+    # shouldn't be anything left in @args
+    warn "Unknown parameter(s): @args" if @args;
 
-    my $source = defined($parms{swaptab})
-                       ? $parms{swaptab}
-                       : defined( $parms{swaptabdir} )
-                           ? $parms{swaptabdir}.'/swaptab.txt'
-                           : $swaptabSource;
+#    croak "You cannot specify both swaptab and swaptabdir parameters"
+#      if ( defined( $parms{swaptab} ) && defined( $parms{swaptabdir} ) );
 
+    my $source =
+        defined( $parms{swaptab} )    ? $parms{swaptab}
+#      : defined( $parms{swaptabdir} ) ? $parms{swaptabdir} . '/swaptab.txt'
+      : $swaptabSource;
 
-    open STAB, $source
-      or croak "Can't read from swap tab source : $source";
+    if ( ( -f "/sbin/swap" || $swaptabSource ne $source ) &&  open STAB, $source ) {
+        while (<STAB>) {
+            chomp;
+            my ( $dev, $devn, $slo, $sbl, $sfree ) = split /\s+/;
+            next if !defined($devn);
+            next if !defined($slo);
+            next if !defined($sbl);
+            next if !defined($sfree);
 
-    while (<STAB>) {
-        chomp;
-        my ( $dev, $devn, $slo, $sbl, $sfree) = split /\s+/;
+            # pass the header line
+            next if $dev eq 'swapfile';
+            next if $dev !~ m!^/!;
+            # strip path to device file
+            if ( $dev =~ m/dsk/ ) {
+                $dev =~ s!.*\/!!;
+                $self->{dev2mp}{$dev} = 'swap';
 
-        # pass the header line
-        next if $dev eq 'swapfile';
-        
-        # strip path to device file
-        if ($dev =~ m/dsk/) {
-            $dev =~ s!.*\/!!;
-            $self->{dev2mp}{$dev} = 'swap';
-            # This is not really a mount point, so don't feed the reverse...
+                # This is not really a mount point, so don't feed the reverse...
+            }
+            $self->{swap}{$dev}{device}  = $dev;
+            $self->{swap}{$dev}{devnum}  = $devn;
+            $self->{swap}{$dev}{swaplo}  = $slo;
+            $self->{swap}{$dev}{swaplen} = $sbl;
+            $self->{swap}{$dev}{free}    = $sfree;
         }
-        $self->{swap}{$dev}{device}  = $dev;
-        $self->{swap}{$dev}{devnum}  = $devn;
-        $self->{swap}{$dev}{swaplo}  = $slo;
-        $self->{swap}{$dev}{swaplen} = $sbl;
-        $self->{swap}{$dev}{free}    = $sfree;
+        close STAB
+          or warn "Can't close from swap tab source : $source";
+    }
+    else {
+        warn "Can't open swap tab source : $source";
     }
-    close STAB;
 }
 
 'Solaris::Disk::Mnttab';
@@ -228,12 +250,12 @@ Jrme Fenal <jfenal@free.fr>
 
 =head1 VERSION
 
-This is version 0.1 of the Solaris::Disk::Mnttab
+This is version 0.03 of the Solaris::Disk::Mnttab
 
 
 =head1 COPYRIGHT
 
-Copyright (C) 2004 Jrme Fenal. All Rights Reserved
+Copyright (C) 2004, 2005 Jrme Fenal. All Rights Reserved
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself.
